]> de.git.xonotic.org Git - xonotic/darkplaces.git/blobdiff - menu.c
Added the togglemenu command for both menus and the chr builtin command (menu qc)
[xonotic/darkplaces.git] / menu.c
diff --git a/menu.c b/menu.c
index d3af61eab726e693ba64fd99286a8ad3821b96a1..0185ad452c277f0cc11e76f7ecce5444ee058a87 100644 (file)
--- a/menu.c
+++ b/menu.c
@@ -1112,7 +1112,7 @@ void M_DrawCheckbox (int x, int y, int on)
 }
 
 
-#define OPTIONS_ITEMS 32
+#define OPTIONS_ITEMS 34
 
 int options_cursor;
 
@@ -1127,6 +1127,7 @@ extern cvar_t snd_staticvolume;
 extern cvar_t gl_delayfinish;
 extern cvar_t slowmo;
 extern dllhandle_t jpeg_dll;
+extern cvar_t gl_texture_anisotropy;
 
 void M_Menu_Options_AdjustSliders (int dir)
 {
@@ -1148,6 +1149,8 @@ void M_Menu_Options_AdjustSliders (int dir)
                Cvar_SetValueQuick (&scr_viewsize, bound(30, scr_viewsize.value + dir * 10, 120));
        else if (options_cursor == optnum++)
                Cvar_SetValueQuick (&scr_screenshot_jpeg, !scr_screenshot_jpeg.integer);
+       else if (options_cursor == optnum++)
+               Cvar_SetValueQuick (&scr_screenshot_jpeg_quality, bound(0, scr_screenshot_jpeg_quality.value + dir * 0.1, 1));
        else if (options_cursor == optnum++)
                Cvar_SetValueQuick (&r_sky, !r_sky.integer);
        else if (options_cursor == optnum++)
@@ -1156,6 +1159,8 @@ void M_Menu_Options_AdjustSliders (int dir)
                Cvar_SetValueQuick (&gl_dither, !gl_dither.integer);
        else if (options_cursor == optnum++)
                Cvar_SetValueQuick (&gl_delayfinish, !gl_delayfinish.integer);
+       else if (options_cursor == optnum++)
+               Cvar_SetValueQuick (&gl_texture_anisotropy, bound(0, gl_texture_anisotropy.value + dir, 8));
        else if (options_cursor == optnum++)
                Cvar_SetValueQuick (&slowmo, bound(0, slowmo.value + dir * 0.25, 5));
        else if (options_cursor == optnum++)
@@ -1211,7 +1216,7 @@ void M_Options_PrintCommand(char *s, int enabled)
 {
        if (opty >= 32)
        {
-               DrawQ_Fill(menu_x, menu_y + opty, 640, 8, optnum == optcursor ? (0.5 + 0.2 * sin(realtime * M_PI)) : 0, 0, 0, 0.5, 0);
+               DrawQ_Fill(menu_x, menu_y + opty, 320, 8, optnum == optcursor ? (0.5 + 0.2 * sin(realtime * M_PI)) : 0, 0, 0, 0.5, 0);
                M_ItemPrint(0, opty, s, enabled);
        }
        opty += 8;
@@ -1222,7 +1227,7 @@ void M_Options_PrintCheckbox(char *s, int enabled, int yes)
 {
        if (opty >= 32)
        {
-               DrawQ_Fill(menu_x, menu_y + opty, 640, 8, optnum == optcursor ? (0.5 + 0.2 * sin(realtime * M_PI)) : 0, 0, 0, 0.5, 0);
+               DrawQ_Fill(menu_x, menu_y + opty, 320, 8, optnum == optcursor ? (0.5 + 0.2 * sin(realtime * M_PI)) : 0, 0, 0, 0.5, 0);
                M_ItemPrint(0, opty, s, enabled);
                M_DrawCheckbox(0 + strlen(s) * 8 + 8, opty, yes);
        }
@@ -1234,7 +1239,7 @@ void M_Options_PrintSlider(char *s, int enabled, float value, float minvalue, fl
 {
        if (opty >= 32)
        {
-               DrawQ_Fill(menu_x, menu_y + opty, 640, 8, optnum == optcursor ? (0.5 + 0.2 * sin(realtime * M_PI)) : 0, 0, 0, 0.5, 0);
+               DrawQ_Fill(menu_x, menu_y + opty, 320, 8, optnum == optcursor ? (0.5 + 0.2 * sin(realtime * M_PI)) : 0, 0, 0, 0.5, 0);
                M_ItemPrint(0, opty, s, enabled);
                M_DrawSlider(0 + strlen(s) * 8 + 8, opty, value, minvalue, maxvalue);
        }
@@ -1253,8 +1258,6 @@ void M_Options_Draw (void)
        p = Draw_CachePic("gfx/p_option.lmp");
        M_DrawPic((320-p->width)/2, 4, "gfx/p_option.lmp");
 
-       // LordHavoc: FIXME: overbright needs to be disabled in GAME_GOODVSBAD2 but combine should not be disabled
-       // LordHavoc: perhaps it's time for Overbright Bits to die, and a r_lightmapintensity option to be added?
        optnum = 0;
        optcursor = options_cursor;
        visible = (vid.conheight - 32) / 8;
@@ -1273,14 +1276,16 @@ void M_Options_Draw (void)
        M_Options_PrintSlider(  "Conback Brightness", true, scr_conbrightness.value, 0, 1);
        M_Options_PrintSlider(  "       Screen size", true, scr_viewsize.value, 30, 120);
        M_Options_PrintCheckbox("  JPEG screenshots", jpeg_dll != NULL, scr_screenshot_jpeg.integer);
+       M_Options_PrintSlider(  "      JPEG quality", jpeg_dll != NULL, scr_screenshot_jpeg_quality.value, 0, 1);
        M_Options_PrintCheckbox("               Sky", true, r_sky.integer);
        M_Options_PrintCheckbox("   Texture Combine", true, gl_combine.integer);
        M_Options_PrintCheckbox("         Dithering", true, gl_dither.integer);
        M_Options_PrintCheckbox("Delay gfx (faster)", true, gl_delayfinish.integer);
+       M_Options_PrintSlider(  "Anisotropic Filter", gl_support_anisotropy, gl_texture_anisotropy.value, 0, 8);
        M_Options_PrintSlider(  "        Game Speed", sv.active, slowmo.value, 0, 5);
-       M_Options_PrintSlider(  "   CD Music Volume", cdaudioinitialized, bgmvolume.value, 0, 1);
-       M_Options_PrintSlider(  "      Sound Volume", snd_initialized, volume.value, 0, 1);
-       M_Options_PrintSlider(gamemode == GAME_GOODVSBAD2 ? "      Music Volume" : "    Ambient Volume", snd_initialized, snd_staticvolume.value, 0, 1);
+       M_Options_PrintSlider(  "   CD Music Volume", cdaudioinitialized.integer, bgmvolume.value, 0, 1);
+       M_Options_PrintSlider(  "      Sound Volume", snd_initialized.integer, volume.value, 0, 1);
+       M_Options_PrintSlider(gamemode == GAME_GOODVSBAD2 ? "      Music Volume" : "    Ambient Volume", snd_initialized.integer, snd_staticvolume.value, 0, 1);
        M_Options_PrintSlider(  "         Crosshair", true, crosshair.value, 0, 5);
        M_Options_PrintSlider(  "    Crosshair Size", true, crosshair_size.value, 1, 5);
        M_Options_PrintCheckbox("  Static Crosshair", true, crosshair_static.integer);
@@ -1357,7 +1362,7 @@ void M_Options_Key (int k, char ascii)
        }
 }
 
-#define        OPTIONS_EFFECTS_ITEMS   20
+#define        OPTIONS_EFFECTS_ITEMS   22
 
 int options_effects_cursor;
 
@@ -1385,6 +1390,7 @@ extern cvar_t cl_particles_sparks;
 extern cvar_t cl_particles_bubbles;
 extern cvar_t cl_particles_blood;
 extern cvar_t cl_particles_blood_alpha;
+extern cvar_t cl_particles_blood_bloodhack;
 
 void M_Menu_Options_Effects_AdjustSliders (int dir)
 {
@@ -1424,10 +1430,14 @@ void M_Menu_Options_Effects_AdjustSliders (int dir)
                Cvar_SetValueQuick (&cl_particles_blood, !cl_particles_blood.integer);
        else if (options_effects_cursor == optnum++)
                Cvar_SetValueQuick (&cl_particles_blood_alpha, bound(0.2, cl_particles_blood_alpha.value + dir * 0.1, 1));
+       else if (options_effects_cursor == optnum++)
+               Cvar_SetValueQuick (&cl_particles_blood_bloodhack, !cl_particles_blood_bloodhack.integer);
        else if (options_effects_cursor == optnum++)
                Cvar_SetValueQuick (&r_lerpmodels, !r_lerpmodels.integer);
        else if (options_effects_cursor == optnum++)
                Cvar_SetValueQuick (&r_lerpsprites, !r_lerpsprites.integer);
+       else if (options_effects_cursor == optnum++)
+               Cvar_SetValueQuick (&r_wateralpha, bound(0, r_wateralpha.value + dir * 0.1, 1));
        else if (options_effects_cursor == optnum++)
                Cvar_SetValueQuick (&r_waterscroll, bound(0, r_waterscroll.value + dir * 0.5, 10));
        else if (options_effects_cursor == optnum++)
@@ -1466,8 +1476,10 @@ void M_Options_Effects_Draw (void)
        M_Options_PrintCheckbox("               Bubbles", true, cl_particles_bubbles.integer);
        M_Options_PrintCheckbox("                 Blood", true, cl_particles_blood.integer);
        M_Options_PrintSlider(  "         Blood Opacity", true, cl_particles_blood_alpha.value, 0.2, 1);
+       M_Options_PrintCheckbox("Force New Blood Effect", true, cl_particles_blood_bloodhack.integer);
        M_Options_PrintCheckbox("   Model Interpolation", true, r_lerpmodels.integer);
        M_Options_PrintCheckbox("  Sprite Interpolation", true, r_lerpsprites.integer);
+       M_Options_PrintSlider(  " Water Alpha (opacity)", true, r_wateralpha.value, 0, 1);
        M_Options_PrintSlider(  "        Water Movement", true, r_waterscroll.value, 0, 10);
        M_Options_PrintSlider(  " GeForce3 Water Shader", true, r_watershader.value, 0, 10);
 }
@@ -1645,8 +1657,8 @@ void M_Options_ColorControl_Draw (void)
        opty = 32 - bound(0, optcursor - (visible >> 1), max(0, OPTIONS_COLORCONTROL_ITEMS - visible)) * 8;
 
        M_Options_PrintCommand( "     Reset to defaults", true);
-       M_Options_PrintCheckbox("Hardware Gamma Control", vid_hardwaregammasupported, v_hwgamma.integer);
-       M_Options_PrintSlider(  "                 Gamma", !v_color_enable.integer && vid_hardwaregammasupported && v_hwgamma.integer, v_gamma.value, 1, 5);
+       M_Options_PrintCheckbox("Hardware Gamma Control", vid_hardwaregammasupported.integer, v_hwgamma.integer);
+       M_Options_PrintSlider(  "                 Gamma", !v_color_enable.integer && vid_hardwaregammasupported.integer && v_hwgamma.integer, v_gamma.value, 1, 5);
        M_Options_PrintSlider(  "              Contrast", !v_color_enable.integer, v_contrast.value, 1, 5);
        M_Options_PrintSlider(  "            Brightness", !v_color_enable.integer, v_brightness.value, 0, 0.8);
        M_Options_PrintCheckbox("  Color Level Controls", true, v_color_enable.integer);
@@ -1654,10 +1666,10 @@ void M_Options_ColorControl_Draw (void)
        M_Options_PrintSlider(  "          Black: Green", v_color_enable.integer, v_color_black_g.value, 0, 0.8);
        M_Options_PrintSlider(  "          Black: Blue ", v_color_enable.integer, v_color_black_b.value, 0, 0.8);
        M_Options_PrintSlider(  "          Black: Grey ", v_color_enable.integer, (v_color_black_r.value + v_color_black_g.value + v_color_black_b.value) / 3, 0, 0.8);
-       M_Options_PrintSlider(  "           Grey: Red  ", v_color_enable.integer && vid_hardwaregammasupported && v_hwgamma.integer, v_color_grey_r.value, 0, 0.95);
-       M_Options_PrintSlider(  "           Grey: Green", v_color_enable.integer && vid_hardwaregammasupported && v_hwgamma.integer, v_color_grey_g.value, 0, 0.95);
-       M_Options_PrintSlider(  "           Grey: Blue ", v_color_enable.integer && vid_hardwaregammasupported && v_hwgamma.integer, v_color_grey_b.value, 0, 0.95);
-       M_Options_PrintSlider(  "           Grey: Grey ", v_color_enable.integer && vid_hardwaregammasupported && v_hwgamma.integer, (v_color_grey_r.value + v_color_grey_g.value + v_color_grey_b.value) / 3, 0, 0.95);
+       M_Options_PrintSlider(  "           Grey: Red  ", v_color_enable.integer && vid_hardwaregammasupported.integer && v_hwgamma.integer, v_color_grey_r.value, 0, 0.95);
+       M_Options_PrintSlider(  "           Grey: Green", v_color_enable.integer && vid_hardwaregammasupported.integer && v_hwgamma.integer, v_color_grey_g.value, 0, 0.95);
+       M_Options_PrintSlider(  "           Grey: Blue ", v_color_enable.integer && vid_hardwaregammasupported.integer && v_hwgamma.integer, v_color_grey_b.value, 0, 0.95);
+       M_Options_PrintSlider(  "           Grey: Grey ", v_color_enable.integer && vid_hardwaregammasupported.integer && v_hwgamma.integer, (v_color_grey_r.value + v_color_grey_g.value + v_color_grey_b.value) / 3, 0, 0.95);
        M_Options_PrintSlider(  "          White: Red  ", v_color_enable.integer, v_color_white_r.value, 1, 5);
        M_Options_PrintSlider(  "          White: Green", v_color_enable.integer, v_color_white_g.value, 1, 5);
        M_Options_PrintSlider(  "          White: Blue ", v_color_enable.integer, v_color_white_b.value, 1, 5);
@@ -1706,7 +1718,7 @@ void M_Options_ColorControl_Key (int k, char ascii)
        switch (k)
        {
        case K_ESCAPE:
-               M_Menu_Main_f ();
+               M_Menu_Options_f ();
                break;
 
        case K_ENTER:
@@ -2142,7 +2154,10 @@ int video_cursor = 0;
 int video_cursor_table[] = {56, 68, 80, 100};
 // note: if modes are added to the beginning of this list, update the
 // video_resolution = x; in M_Menu_Video_f below
-unsigned short video_resolutions[][2] = {{320,240}, {400,300}, {512,384}, {640,480}, {800,600}, {1024,768}, {1152,864}, {1280,960}, {1280,1024}, {1600,1200}, {1792,1344}, {1920,1440}, {2048,1536}};
+unsigned short video_resolutions[][2] = {{320,240}, {400,300}, {512,384}, {640,480}, {800,600}, {1024,768}, {1152,864}, {1280,960}, {1280,1024}, {1600,1200}, {1792,1344}, {1920,1440}, {2048,1536}, {0,0}};
+// this is the number of the 640x480 mode in the list
+#define VID_640 3
+#define VID_RES_COUNT ((int)(sizeof(video_resolutions) / sizeof(video_resolutions[0])) - 1)
 int video_resolution;
 
 extern int current_vid_fullscreen;
@@ -2158,18 +2173,18 @@ void M_Menu_Video_f (void)
        m_entersound = true;
 
        // Look for the current resolution
-       for (video_resolution = 0; video_resolution < (int) (sizeof (video_resolutions) / sizeof (video_resolutions[0])); video_resolution++)
+       for (video_resolution = 0; video_resolution < VID_RES_COUNT; video_resolution++)
        {
                if (video_resolutions[video_resolution][0] == current_vid_width &&
                        video_resolutions[video_resolution][1] == current_vid_height)
                        break;
        }
 
-       // Default to 800x600 if we didn't find it
-       if (video_resolution == sizeof (video_resolutions) / sizeof (video_resolutions[0]))
+       // Default to VID_640 if we didn't find it
+       if (video_resolution == VID_RES_COUNT)
        {
                // may need to update this number if mode list changes
-               video_resolution = 4;
+               video_resolution = VID_640;
                Cvar_SetValueQuick (&vid_width, video_resolutions[video_resolution][0]);
                Cvar_SetValueQuick (&vid_height, video_resolutions[video_resolution][1]);
        }
@@ -2218,10 +2233,10 @@ void M_Menu_Video_AdjustSliders (int dir)
                case 0:
                {
                        int new_resolution = video_resolution + dir;
-                       if (new_resolution < 0)
-                               video_resolution = sizeof (video_resolutions) / sizeof (video_resolutions[0]) - 1;
-                       else if (new_resolution > (int) (sizeof (video_resolutions) / sizeof (video_resolutions[0]) - 1))
-                               video_resolution = 0;
+                       if (gamemode == GAME_FNIGGIUM ? new_resolution < VID_640 : new_resolution < 0)
+                               video_resolution = VID_RES_COUNT - 1;
+                       else if (new_resolution > VID_RES_COUNT)
+                               video_resolution = gamemode == GAME_FNIGGIUM ? VID_640 : 0;
                        else
                                video_resolution = new_resolution;
 
@@ -3483,8 +3498,6 @@ void M_Init (void)
        menuplyr_load = true;
        menuplyr_pixels = NULL;
 
-       Cmd_AddCommand ("togglemenu", M_ToggleMenu_f);
-
        Cmd_AddCommand ("menu_main", M_Menu_Main_f);
        Cmd_AddCommand ("menu_singleplayer", M_Menu_SinglePlayer_f);
        Cmd_AddCommand ("menu_load", M_Menu_Load_f);
@@ -3867,10 +3880,10 @@ void MP_Init (void)
 
        // set time
        *prog->time = realtime;
-       
+
        // call the prog init
        PRVM_ExecuteProgram((func_t) (PRVM_ED_FindFunction(M_F_INIT) - prog->functions),"");
-       
+
        PRVM_End;
 }
 
@@ -3897,7 +3910,7 @@ void MR_SetRouting(qboolean forceold)
                MR_Draw = M_Draw;
                MR_ToggleMenu_f = M_ToggleMenu_f;
                MR_Shutdown = M_Shutdown;
-               
+
                // init
                if(!m_init)
                {
@@ -3914,7 +3927,7 @@ void MR_SetRouting(qboolean forceold)
                MR_Draw = MP_Draw;
                MR_ToggleMenu_f = MP_ToggleMenu_f;
                MR_Shutdown = MP_Shutdown;
-               
+
                if(!mp_init)
                {
                        MP_Init();
@@ -3931,21 +3944,28 @@ void MR_Restart(void)
        MR_SetRouting (FALSE);
 }
 
+void Call_MR_ToggleMenu_f(void)
+{
+       if(MR_ToggleMenu_f)
+               MR_ToggleMenu_f();
+}
+
 void MR_Init()
 {
        // set router console commands
        Cvar_RegisterVariable (&forceqmenu);
        Cmd_AddCommand ("menu_restart",MR_Restart);
+       Cmd_AddCommand ("togglemenu", Call_MR_ToggleMenu_f);
 
        // use -forceqmenu to use always the normal quake menu (it sets forceqmenu to 1)
        if(COM_CheckParm("-forceqmenu"))
                Cvar_SetValueQuick(&forceqmenu,1);
-       // use -useqmenu for debugging proposes, cause it starts 
+       // use -useqmenu for debugging proposes, cause it starts
        // the normal quake menu only the first time
        else if(COM_CheckParm("-useqmenu"))
                MR_SetRouting (TRUE);
        else
-               MR_SetRouting (FALSE);  
+               MR_SetRouting (FALSE);
 }